import requests
import json
import logging
import os
import subprocess

class Order:
    def __init__(self, order_info):
        self.id = order_info.get('id')
        self.status_order_name = order_info.get('status_order_name')
        self.shipping_tracking = order_info.get('shipping_tracking')
        self.comments_customer = order_info.get('comments_customer')

    def has_changes(self, other_order):
        return (
            self.status_order_name != other_order.status_order_name or
            self.shipping_tracking != other_order.shipping_tracking
        )

class OrderFetcher:
    EMAIL = 'michal@shumee.pl'
    TOKEN = '149dd9e9-665b-45bb-9a60-14bc4c598720'

    def __init__(self, email=None, token=None):
        self.email = email or self.EMAIL
        self.token = token or self.TOKEN
        self.url = 'https://b2b.vidaxl.com/api_customer/orders.json'
        self.output_file = 'output_selected.json'
        self.new_file = 'output_new.json'
        self.api_orders_file = 'api_orders.json'

    def fetch_orders(self):
        # Clear output_new.json at the beginning
        self._clear_file(self.new_file)

        # Clear api_orders.json at the beginning
        self._clear_file(self.api_orders_file)

        try:
            r = self._make_request()

            if r.status_code == 200:
                response_data = json.loads(r.text)
                selected_data = self._process_orders(response_data)
                self._update_existing_orders(selected_data)
                self._map_and_save_api_orders(response_data)
                self._copy_api_to_selected()
            else:
                logging.error(f"Error: {r.status_code} - {r.text}")
        except requests.RequestException as e:
            logging.error(f"Request error: {e}")
        except json.JSONDecodeError as e:
            logging.error(f"Error decoding JSON: {e}")

    def _make_request(self):
        return requests.get(self.url, auth=requests.auth.HTTPBasicAuth(username=self.email, password=self.token))

    def _process_orders(self, response_data):
        orders = [Order(order_info.get('order', {})) for order_info in response_data]
        return [{
            'id': order.id,
            'status_order_name': order.status_order_name,
            'shipping_tracking': order.shipping_tracking,
            'comments_customer': order.comments_customer
        } for order in orders]

    def _update_existing_orders(self, new_orders):
        existing_orders = self._get_existing_orders()
        existing_ids = {order['id'] for order in existing_orders}
        orders_by_comments_customer = {}

        # Przetwarzanie nowych zamówień
        new_orders_to_save = []
        for order in new_orders:
            if order['id'] not in existing_ids or self._order_has_changes(order, existing_orders):
                change_id = self._get_change_id(order, existing_orders)
                order['change_id'] = change_id

                # Sprawdzenie czy już istnieje zamówienie z takim comments_customer
                if order['comments_customer'] in orders_by_comments_customer:
                    # Jeśli tak, scal zamówienia
                    existing_order = orders_by_comments_customer[order['comments_customer']]
                    
                    # Sprawdzenie czy wartość jest None przed użyciem operatora +=
                    if existing_order['status_order_name'] is not None:
                        existing_order['status_order_name'] += f",{order['status_order_name']}"
                    else:
                        existing_order['status_order_name'] = order['status_order_name']

                    if existing_order['shipping_tracking'] is not None:
                        existing_order['shipping_tracking'] += f",{order['shipping_tracking']}"
                    else:
                        existing_order['shipping_tracking'] = order['shipping_tracking']

                    # Dopisz nową wartość do change_id tylko jeśli jest różna od tej w pierwszym zamówieniu
                    if existing_order['change_id'] != order['change_id']:
                        existing_order['change_id'] += f",{order['change_id']}"
                    
                else:
                    # Jeśli nie, dodaj do słownika
                    orders_by_comments_customer[order['comments_customer']] = order
                    new_orders_to_save.append(order)

            self._save_to_file(self.new_file, new_orders_to_save)

            # Update the existing file
            existing_orders.extend(new_orders_to_save)
            self._save_to_file(self.output_file, existing_orders)

    def _order_has_changes(self, new_order, existing_orders):
        for existing_order in existing_orders:
            if existing_order['id'] == new_order['id']:
                existing = Order(existing_order)
                new = Order(new_order)
                return existing.has_changes(new)
        return False

    def _get_change_id(self, new_order, existing_orders):
        for existing_order in existing_orders:
            if existing_order['id'] == new_order['id']:
                if existing_order['status_order_name'] != new_order['status_order_name']:
                    if new_order['status_order_name'] == 'Sent' and existing_order.get('address') != "1 MAJA 31/33 M6":
                        return 3  # Status change to "Sent"
                    return 2  # Other status change
                elif existing_order['shipping_tracking'] != new_order['shipping_tracking']:
                    return 3  # Tracking change
                elif existing_order.get('address') == "1 MAJA 31/33 M6":
                    return 4  # Special case for "address" value
        return 1  # New order


    def _get_existing_orders(self):
        if os.path.exists(self.output_file):
            with open(self.output_file, 'r') as file:
                return json.load(file)
        return []

    def _save_to_file(self, filename, orders):
        with open(filename, 'w') as output_file:
            json.dump(orders, output_file, indent=2)
        logging.info(f"Orders updated in '{filename}'")

    def _map_and_save_api_orders(self, api_response):
        api_orders = self._map_orders(api_response)
        self._save_to_file(self.api_orders_file, api_orders)
        logging.info(f"API orders saved to '{self.api_orders_file}'")

    def _copy_api_to_selected(self):
        # Load api_orders.json
        api_orders = self._get_api_orders()

        # Save api_orders.json to output_selected.json, overwriting existing data
        self._save_to_file(self.output_file, api_orders)
        logging.info(f"API orders copied to '{self.output_file}'")

    def _map_orders(self, response_data):
        orders = [Order(order_info.get('order', {})) for order_info in response_data]
        return [{
            'id': order.id,
            'status_order_name': order.status_order_name,
            'shipping_tracking': order.shipping_tracking,
            'comments_customer': order.comments_customer
        } for order in orders]

    def _clear_file(self, filename):
        with open(filename, 'w'):
            pass

    def _get_api_orders(self):
        if os.path.exists(self.api_orders_file):
            with open(self.api_orders_file, 'r') as file:
                return json.load(file)
        return []

def main():
    order_fetcher = OrderFetcher()
    order_fetcher.fetch_orders()

    second_script_path = 'ApiloSender.py'
    subprocess.run(['python', second_script_path])

if __name__ == "__main__":
    main()
